# -*- coding: utf-8 -*-
## PloneBooking: Online Booking Tool to allow booking on any kind of ressource
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
    PloneBooking: Booking content workflow creation
"""

__author__  = ''
__docformat__ = 'restructuredtext'



__version__ = "$Revision: 1.12 $"[11:-2]

from Products.CMFCore.WorkflowTool import addWorkflowFactory

from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.PythonScripts.PythonScript import manage_addPythonScript

def setupBooking_workflow(wf):
    """
    Booking Worklow contains 2 scripts :
      - notify
      - delete
    """

    # create script notify
    manage_addPythonScript(wf.scripts, 'notify')
    script = getattr(wf.scripts, 'notify')
    script.write("""##parameters=state_change""")
    script.write("""context.booking_workflow_notification(state_change.object)""")

    # create script delete
    manage_addPythonScript(wf.scripts, 'delete')
    script = getattr(wf.scripts, 'delete')
    script.write("""##parameters=state_change""")
    script.write("""context.booking_workflow_delete(state_change)""")
    
    wf.setProperties(title='Default Workflow  for Booking')

    for s in ['canceled', 'booked', 'pending']:
        wf.states.addState(s)
    for t in ['cancel', 'book', 'submit', 'auto_book']:
        wf.transitions.addTransition(t)
    for v in ['action', 'review_history', 'comments', 'actor', 'time']:
        wf.variables.addVariable(v)
    for l in ['reviewer_queue']:
        wf.worklists.addWorklist(l)
    for p in ('Access contents information', 'Modify portal content', 'View'):
        wf.addManagedPermission(p)
        

    ## Initial State
    wf.states.setInitialState('pending')

    ## States initialization
    sdef = wf.states['canceled']
    sdef.setProperties(title="""Canceled""",
                       transitions=('submit',))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 0, ['Manager', 'Owner', 'Reviewer'])

    sdef = wf.states['booked']
    sdef.setProperties(title="""Booked""",
                       transitions=('cancel',))
    sdef.setPermission('Access contents information', 1, ['Manager'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner'])
    sdef.setPermission('View', 1, ['Manager'])

    sdef = wf.states['pending']
    sdef.setProperties(title="""Waiting for reviewer""",
                       transitions=('auto_book', 'book', 'cancel'))
    sdef.setPermission('Access contents information', 1, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Reviewer', 'Owner'])
    sdef.setPermission('View', 1, ['Manager', 'Owner', 'Reviewer'])


    ## Transitions initialization
    tdef = wf.transitions['cancel']
    tdef.setProperties(title="""Member retracts booking""",
                       new_state_id="""canceled""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""delete""",
                       actbox_name="""Retract""",
                       actbox_url="""%(content_url)s/content_retract_form""",
                       actbox_category="""workflow""",
                       props={'guard_permissions': 'Request review'},
                       )

    tdef = wf.transitions['book']
    tdef.setProperties(title="""Book""",
                       new_state_id="""booked""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Book""",
                       actbox_url="""""",
                       actbox_category="""workflow""",
                       props={'guard_permissions': 'Review portal content'},
                       )

    tdef = wf.transitions['submit']
    tdef.setProperties(title="""Member requests booking""",
                       new_state_id="""pending""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Submit""",
                       actbox_url="""%(content_url)s/content_submit_form""",
                       actbox_category="""workflow""",
                       props={'guard_permissions': 'Request review'},
                       )

    tdef = wf.transitions['auto_book']
    tdef.setProperties(title="""automatic book""",
                       new_state_id="""booked""",
                       trigger_type=0,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""""",
                       actbox_url="""""",
                       actbox_category="""workflow""",
                       props={'guard_expr': 'python: not here.isTemporary() and here.publishAutomatically()', 'guard_roles': 'Owner; Member'},
                       )

    ## State Variable
    wf.variables.setStateVar('review_state')

    ## Variables initialization
    vdef = wf.variables['action']
    vdef.setProperties(description="""The last transition""",
                       default_value="""""",
                       default_expr="""transition/getId|nothing""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['review_history']
    vdef.setProperties(description="""Provides access to workflow history""",
                       default_value="""""",
                       default_expr="""state_change/getHistory""",
                       for_catalog=0,
                       for_status=0,
                       update_always=0,
                       props={'guard_permissions': 'Request review; Review portal content'})

    vdef = wf.variables['comments']
    vdef.setProperties(description="""Comments about the last transition""",
                       default_value="""""",
                       default_expr="""python:state_change.kwargs.get('comment', '')""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['actor']
    vdef.setProperties(description="""The ID of the user who performed the last transition""",
                       default_value="""""",
                       default_expr="""user/getId""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['time']
    vdef.setProperties(description="""Time of the last transition""",
                       default_value="""""",
                       default_expr="""state_change/getDateTime""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    ## Worklists Initialization
    ldef = wf.worklists['reviewer_queue']
    ldef.setProperties(description="""Reviewer tasks""",
                       actbox_name="""Pending (%(count)d)""",
                       actbox_url="""%(portal_url)s/search?review_state=pending""",
                       actbox_category="""global""",
                       props={'guard_permissions': 'Review portal content', 'var_match_review_state': 'pending'})


def createBooking_workflow(id):
    "..."
    ob = DCWorkflowDefinition(id)
    setupBooking_workflow(ob)
    return ob

addWorkflowFactory(createBooking_workflow,
                   id='booking_workflow',
                   title='Default Workflow  for Booking')

    