# -*- coding: utf-8 -*-
## PloneBooking: Online Booking Tool to allow booking on any kind of ressource
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
    PloneBooking: Booking Center
"""

__version__ = "$Revision: 1.9 $"
__author__  = ''
__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo
from AccessControl import Unauthorized
from DateTime import DateTime

# CMF imports
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName

# Archetypes imports
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

# PloneBooking imports
from Products.PloneBooking.config import PROJECTNAME
from Products.PloneBooking.content.schemata import BookingCenterSchema
from Products.PloneBooking.interfaces.interfaces import IBookingCenter
from Products.PloneBooking import BookingPermissions

class BookingCenter(BaseFolder):
    """
      Booking Center contains all Bookable Object and Bookings
    """
    __implements__ = (BaseFolder.__implements__, IBookingCenter, )

    meta_type      = 'BookingCenter'
    archetype_name = 'Booking center'

    default_view = 'bookingcenter_view'
    immediate_view = 'bookingcenter_view'

    global_allow = 1
    filter_content_types = 1
    allowed_content_types = (
        'BookableObject',
    )

    schema = BookingCenterSchema

    actions = (
        {'id': 'view',
         'name': 'View',
         'action': 'string:${object_url}/bookingcenter_view',
         'permissions': (CMFCorePermissions.View,)
        },
        {
        'id'          : 'local_roles',
        'name'        : 'Sharing',
        'action'      : 'string:${object_url}/folder_localrole_form',
        'permissions' : (CMFCorePermissions.ManageProperties,),
         },
    )

#    actions = (
#        {
#        'id'          : 'local_roles',
#        'name'        : 'Sharing',
#        'action'      : 'string:${object_url}/folder_localrole_form',
#        'permissions' : (CMFCorePermissions.ManageProperties,),
#         },
#        {'id': 'add_booking',
#         'name': 'Add booking',
#         'condition': 'python:object.canBook()',
#         'action': 'string:${object/getBookingContainerUrl}/createObject?type_name=Booking',
#         'permissions': (BookingPermissions.AddBooking,),
#         'category' : 'object_booking_actions'
#        },
#        {'id': 'add_bookableobject',
#         'name': 'Add bookable object',
#         'action': 'string:${object/getBookableObjectContainerUrl}/createObject?type_name=BookableObject',
#         'permissions': (BookingPermissions.AddBookableObject,),
#         'category' : 'object_booking_actions'
#        },
#        {'id': 'display_bookings_listing',
#         'name': 'Display all bookings listing',
#         'action': 'string:${object/getBookingContainerUrl}',
#         'permissions': (CMFCorePermissions.View,),
#         'category' : 'object_booking_actions'
#        },
#        {'id': 'display_bookableobjects',
#         'name': 'Display all bookable objects',
#         'action': 'string:${object/getBookableObjectContainerUrl}',
#         'permissions': (CMFCorePermissions.View,),
#         'category' : 'object_booking_actions',
#         'visible' : 0
#        },
#        )

    security = ClassSecurityInfo()

    security.declarePrivate('getTypeDisplayList')
    def getTypeDisplayList(self, ):
        """Returns all types as a DisplayList"""
        
        return DisplayList([(x, x) for x in self.getTypes()])
    
    security.declarePrivate('getCategoryDisplayList')
    def getCategoryDisplayList(self, ):
        """Returns all categories as a DisplayList"""
        
        return DisplayList([(x, x) for x in self.getCategories()])

    security.declarePublic('getBookingCenter')
    def getBookingCenter(self):
        """Returns booking center himself"""

        utool = getToolByName(self, 'portal_url')
        return self.restrictedTraverse(utool.getRelativeContentURL(self))

    security.declareProtected(CMFCorePermissions.View, 'getBookings')
    def getBookings(self, start_date=None, end_date=None, **kwargs):
        """Get booking objects in a interval of time (start_date, end_date).
        
        @param start_date: start date of scan. This is a DateTime
        @param end_date: end date of scan. This is a DateTime
        """

        return [x.getObject() for x in self.getBookingBrains(start_date, end_date, **kwargs)]

    security.declareProtected(CMFCorePermissions.View, 'getBookingBrains')
    def getBookingBrains(self, start_date=None, end_date=None,  **kwargs):
        """Get booking brains in a interval of time (start_date, end_date).
        
        @param start_date: start date of scan. This is a DateTime
        @param end_date: end date of scan. This is a DateTime
        """
        
        # Initialize
        ctool = getToolByName(self, 'portal_catalog')
        btool = getToolByName(self, 'portal_booking')
        center_obj = self.getBookingCenter()
        center_path = '/'.join(center_obj.getPhysicalPath())
        query_args = {}
        start_ts = None
        end_ts = None
        if start_date is not None:
            start_ts = btool.zdt2ts(start_date)
        if end_date is not None:
            end_ts = btool.zdt2ts(end_date)

        # Add default query args
        query_args['path'] = center_path
        query_args['portal_type'] = 'Booking'
        query_args['sort_on'] = 'start'
        
        # Add query args specific to start date and end date        
        start_end_ranges = []
        if start_date is not None:
            # Start date between booking start and end date
            start_end_ranges.append((start_date, 'max', start_date, 'min'))
        if end_date is not None:
            # End date between booking start and end date
            start_end_ranges.append((end_date, 'max', end_date, 'min'))
        # Booking in start date and end date
        start_end_ranges.append((start_date, 'min', end_date, 'max'))

        # Update query_args
        if kwargs:
            query_args.update(kwargs)

        # Get brains
        brains = []
        brain_rids = []
        for sdate, srange, edate, erange in start_end_ranges:
            query_args_copy = query_args.copy()
            if sdate is not None:
                query_args_copy['start'] = {'query' : sdate, 'range' : srange}

            if edate is not None:
                query_args_copy['end'] = {'query' : edate, 'range' : erange}
            
            new_brains = ctool.searchResults(**query_args_copy)
            for brain in new_brains:
                brain_rid = brain.getRID()
                
                if brain_rid in brain_rids:
                    continue
                
                brain_rids.append(brain_rid)                
                brain_start_dt = DateTime(brain.start)
                brain_start_ts = btool.zdt2ts(brain_start_dt)
                if brain_start_ts == end_ts:
                    continue
                brain_end_dt = DateTime(brain.end)
                brain_end_ts = btool.zdt2ts(brain_end_dt)
                if brain_end_ts == start_ts:
                    continue
                brains.append(brain)
        return brains

    def _getBookingStructure(self, booking_brain, default_title):
        """Build dictionnary with needed information :
        id, title, description, refs, url, start, end, creator, review_state"""

        booking_info = {}
        booking_info['id'] = booking_brain.getId
        title = booking_brain.Title
        booking_info['title'] = title
        if not title:
            booking_info['title'] = default_title
        booking_info['description'] = booking_brain.Description
        booking_info['bookedObjectUID'] = booking_brain.getBookedObjectUID
        booking_info['url'] = booking_brain.getURL()
        booking_info['start'] = DateTime(booking_brain.start)
        booking_info['end'] = DateTime(booking_brain.end)
        booking_info['creator'] = booking_brain.Creator
        booking_info['review_state'] = booking_brain.review_state
        return booking_info
        
    security.declareProtected(CMFCorePermissions.View, 'groupBookingsByIntervalOfMinutes')
    def groupBookingsByIntervalOfMinutes(self, start_date, end_date, interval=30, **kwargs):
        """Group bookings by interval of minutes.
        A brain can exist in several intervals.
        Returns tuple : (group_keys, booking_groups).
        Group key is : (year, month, day, interval). 
        if interval equals 30, Interval of 0 means : 0-30
        if interval equals 30, Interval of 30 means : 30-60
        
        @param start_date: start date of scan. This is a DateTime
        @param end_date: end date of scan. This is a DateTime
        @param interval: interval in minutes. It must be a multiple or a divider of 60
        """
        
        # Interval must be a multiple or a divider of 60. Check it
        if (interval < 60 and (60/interval)*interval != 60) or \
           (interval > 60 and (interval/60)*interval != 60):
            raise ValueError, "Interval must be a multiple or a divider of 60"
        
        # Interval can't be greater than 3600 minutes (a day)
        if interval > 3600:
            raise ValueError, "Interval can't be greater than 3600 minutes"
        
        # Initialize
        btool = getToolByName(self, 'portal_booking')
        default_title = btool.getBookingDefaultTitle()
        group_keys = btool.getIntervalOfMinutesGroupKeys(start_date, end_date, interval)
        booking_groups = {}
        booking_brains = self.getBookingBrains(start_date, end_date, **kwargs)

        # Store brains in booking groups
        for brain in booking_brains:
            # Get brain group keys
            brain_start_date = DateTime(brain.start)
            brain_end_date = DateTime(brain.end)
            if start_date.greaterThanEqualTo(brain_start_date):
                brain_start_date = start_date
            if brain_end_date.greaterThanEqualTo(end_date):
                brain_end_date = end_date
            brain_group_keys = btool.getIntervalOfMinutesGroupKeys(brain_start_date, brain_end_date, interval)

            # Wrap booking
            booking_info = self._getBookingStructure(brain, default_title)
            
            # Append to booking groups
            for key in brain_group_keys:
                value = booking_info.copy()
                if not booking_groups.has_key(key):
                    booking_groups[key] = []
                booking_info['group_by'] = key
                booking_groups[key].append(booking_info)
        
        return group_keys, booking_groups

    security.declareProtected(CMFCorePermissions.View, 'groupBookingsByDay')
    def groupBookingsByDay(self, start_date, end_date, **kwargs):
        """Group bookings by day.
        A brain can exist in several days.
        Returns tuple : (group_keys, booking_groups).
        Group key is : (year, month, day) 
        
        @param start_date: start date of scan. This is a DateTime
        @param end_date: end date of scan. This is a DateTime
        """
        
        # Initialize
        btool = getToolByName(self, 'portal_booking')
        default_title = btool.getBookingDefaultTitle()
        group_keys = btool.getDayGroupKeys(start_date, end_date)
        booking_groups = {}
        booking_brains = self.getBookingBrains(start_date, end_date, **kwargs)

        # Store brains in booking groups
        for brain in booking_brains:
            # Get brain group keys
            brain_start_date = DateTime(brain.start)
            brain_end_date = DateTime(brain.end)
            if start_date.greaterThanEqualTo(brain_start_date):
                brain_start_date = start_date
            if brain_end_date.greaterThanEqualTo(end_date):
                brain_end_date = end_date
            brain_group_keys = btool.getDayGroupKeys(brain_start_date, brain_end_date)
            
            # Wrap booking
            booking_info = self._getBookingStructure(brain, default_title)
            
            # Append to booking groups
            for key in brain_group_keys:
                value = booking_info.copy()
                if not booking_groups.has_key(key):
                    booking_groups[key] = []
                booking_info['group_by'] = key
                booking_groups[key].append(booking_info)

        return group_keys, booking_groups
        
    security.declareProtected(CMFCorePermissions.View, 'groupBookingsByWeek')
    def groupBookingsByWeek(self, start_date, end_date, **kwargs):
        """Group bookings by week.
        A brain can exist in several weeks.
        Returns tuple : (group_keys, booking_groups).
        Group key is : (year, week) 
        
        @param start_date: start date of scan. This is a DateTime
        @param end_date: end date of scan. This is a DateTime
        """
        
        # Initialize
        btool = getToolByName(self, 'portal_booking')
        default_title = btool.getBookingDefaultTitle()
        group_keys = btool.getWeekGroupKeys(start_date, end_date)
        booking_groups = {}
        booking_brains = self.getBookingBrains(start_date, end_date, **kwargs)

        # Store brains in booking groups
        for brain in booking_brains:
            # Get brain group keys
            brain_start_date = DateTime(brain.start)
            brain_end_date = DateTime(brain.end)
            if start_date.greaterThanEqualTo(brain_start_date):
                brain_start_date = start_date
            if brain_end_date.greaterThanEqualTo(end_date):
                brain_end_date = end_date
            brain_group_keys = btool.getWeekGroupKeys(brain_start_date, brain_end_date)
            
            # Wrap booking
            booking_info = self._getBookingStructure(brain, default_title)
            
            # Append to booking groups
            for key in brain_group_keys:
                value = booking_info.copy()
                if not booking_groups.has_key(key):
                    booking_groups[key] = []
                booking_info['group_by'] = key
                booking_groups[key].append(booking_info)

        return group_keys, booking_groups
    
    security.declareProtected(CMFCorePermissions.View, 'groupBookingsByMonth')
    def groupBookingsByMonth(self, start_date, end_date, **kwargs):
        """Group bookings by month.
        A brain can exist in several months.
        Returns tuple : (group_keys, booking_groups).
        Group key is : (year, month) 
        
        @param start_date: start date of scan. This is a DateTime
        @param end_date: end date of scan. This is a DateTime
        """
        
        # Initialize
        btool = getToolByName(self, 'portal_booking')
        default_title = btool.getBookingDefaultTitle()
        group_keys = btool.getMonthGroupKeys(start_date, end_date)
        booking_groups = {}
        booking_brains = self.getBookingBrains(start_date, end_date, **kwargs)

        # Store brains in booking groups
        for brain in booking_brains:
            # Get brain group keys
            brain_start_date = DateTime(brain.start)
            brain_end_date = DateTime(brain.end)
            if start_date.greaterThanEqualTo(brain_start_date):
                brain_start_date = start_date
            if brain_end_date.greaterThanEqualTo(end_date):
                brain_end_date = end_date
            brain_group_keys = btool.getMonthGroupKeys(brain_start_date, brain_end_date)
            
            # Wrap booking
            booking_info = self._getBookingStructure(brain, default_title)
            
            # Append to booking groups
            for key in brain_group_keys:
                value = booking_info.copy()
                if not booking_groups.has_key(key):
                    booking_groups[key] = []
                booking_info['group_by'] = key
                booking_groups[key].append(booking_info)

        return group_keys, booking_groups

    security.declareProtected(CMFCorePermissions.View, 'groupBookingsByYear')
    def groupBookingsByYear(self, start_date, end_date, **kwargs):
        """Group bookings by year.
        A brain can exist in several years.
        Returns tuple : (group_keys, booking_groups).
        Group key is : year
        
        @param start_date: start date of scan. This is a DateTime
        @param end_date: end date of scan. This is a DateTime
        """
        
        # Initialize
        btool = getToolByName(self, 'portal_booking')
        default_title = btool.getBookingDefaultTitle()
        group_keys = btool.getYearGroupKeys(start_date, end_date)
        booking_groups = {}
        booking_brains = self.getBookingBrains(start_date, end_date, **kwargs)

        # Store brains in booking groups
        for brain in booking_brains:
            # Get brain group keys
            brain_start_date = DateTime(brain.start)
            brain_end_date = DateTime(brain.end)
            if start_date.greaterThanEqualTo(brain_start_date):
                brain_start_date = start_date
            if brain_end_date.greaterThanEqualTo(end_date):
                brain_end_date = end_date
            brain_group_keys = btool.getYearGroupKeys(start_date, end_date)
            
            # Wrap booking
            booking_info = self._getBookingStructure(brain, default_title)
            
            # Append to booking groups
            for key in brain_group_keys:
                value = booking_info.copy()
                if not booking_groups.has_key(key):
                    booking_groups[key] = []
                booking_info['group_by'] = key
                booking_groups[key].append(booking_info)

        return group_keys, booking_groups

    def canBook(self):
        """Check for published bookable objects"""

        brains = self.getBookableObjectBrains(review_state='published')

        if not brains:
            return False

        return True

    security.declareProtected(CMFCorePermissions.View, 'getBookableObjectBrains')
    def getBookableObjectBrains(self, **kwargs):
        """Returns all bookable object brains
        """
        
        # Initialize defaults
        ctool = getToolByName(self, 'portal_catalog')
        center_obj = self.getBookingCenter()
        center_path = '/'.join(center_obj.getPhysicalPath())
        query_args = {}
        query_args['path'] = center_path
        query_args['portal_type'] = 'BookableObject'
        
        # Update query args
        if kwargs:
            query_args.update(kwargs)
        
        return ctool.searchResults(**query_args)    
    
    security.declareProtected(CMFCorePermissions.View, 'getBookableObjects')
    def getBookableObjects(self, **kwargs):
        """Returns all bookable objects
        """
        objects = [x.getObject() for x in self.getBookableObjectBrains(**kwargs)]
        objects.sort(lambda x,y: cmp(x.Title(), y.Title()))
        return objects
    
    security.declareProtected(CMFCorePermissions.View, 'getBookableObjectCategories')
    def getBookableObjectCategories(self, **kwargs):
        """Returns the categories used in bookable objects"""
        
        brains = self.getBookableObjectBrains(**kwargs)
        brain_categories = [x.getCategory for x in brains]
        categories = [x for x in self.getCategories() if x in brain_categories]
        categories.sort()
        return categories
 
    security.declareProtected(CMFCorePermissions.View, 'getBookableObjectTypes')
    def getBookableObjectTypes(self, **kwargs):
        """Returns the types used in bookable objects"""
        
        brains = self.getBookableObjectBrains(**kwargs)
        brain_types = [x.getType for x in brains]
        types = [x for x in self.getTypes() if x in brain_types]
        types.sort()
        return types
 
    # Colors features
    security.declareProtected('View', 'getBookingCenterColor')
    def getBookingCenterColor(self, state='booked', type='text'):
        """
        Return the color define on booking center or None
        type in ('text', 'background')
        state in ('pending', 'booked')
        """
        if state == 'booked':
            colors = self.getBookedColors()
        else:
            colors = self.getPendingColors()
            
        if colors.find(':') == -1:
            return None

        lst = colors.split(':')
        if type == 'text':
            return lst[0]
        
        return lst[1]

registerType(BookingCenter, PROJECTNAME)