# -*- coding: utf-8 -*-
## PloneBooking: Online Booking Tool to allow booking on any kind of ressource
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
    PloneBooking: Schemas
"""

__version__ = "$Revision: 1.7 $"
__author__  = ''
__docformat__ = 'restructuredtext'

# CMF imports
from Products.CMFCore import CMFCorePermissions

# Archetypes imports
from Products.Archetypes.public import *

#PloneBooking imports
from Products.PloneBooking.config import I18N_DOMAIN
from Products.PloneBooking.content.vocabulary import \
    CALENDAR_VIEWS, \
    VIEW_MODES, \
    LISTING_VIEWS, \
    BOOKING_REVIEW_MODES, \
    GLOBAL_BOOKING_REVIEW_MODES

BookingCenterSchema = BaseFolderSchema.copy() + Schema((
    
    LinesField(
        'types',                        
        required=True,
        widget=LinesWidget(
            label='Types',
            description='You can define here a list of bookable object types (1 by line)',
            description_msgid='help_bookingcenter_types',
            label_msgid='label_bookingcenter_types',
            i18n_domain= I18N_DOMAIN,
            ),
        ),

    LinesField(
        'categories',
        required=False,
        widget=LinesWidget(
            label='Categories',
            description=' ',
            description_msgid='help_bookingcenter_categories',
            label_msgid='label_bookingcenter_categories',
            i18n_domain=I18N_DOMAIN,
            ),
        ),

    StringField(
        'defaultViewMode',
        required=True,
        default='calendar',
        vocabulary=VIEW_MODES,
        widget=SelectionWidget(
            format='radio',
            label='Default view mode',
            label_msgid='label_bookingcenter_default_view_mode',
            i18n_domain=I18N_DOMAIN,
            ),
        ),
    
    LinesField(
        'availableViewModes',
        required=True,
        default=('listing', 'calendar'),
        vocabulary=VIEW_MODES,
        widget=MultiSelectionWidget(
            format='select',
            label='Available view modes',
            label_msgid='label_bookingcenter_available_view_modes',
            i18n_domain=I18N_DOMAIN,
            ),
        ),
    
    StringField(
        'defaultListingView',
        required=True,
        default='day',
        vocabulary=LISTING_VIEWS,
        widget=SelectionWidget(
            format='select',
            label='Default listing view',
            label_msgid='label_bookingcenter_default_listing_view',
            i18n_domain=I18N_DOMAIN,
            ),
        ),

    LinesField(
        'availableListingViews',
        required=True,
        default=('day', 'week', 'month', 'year'),
        vocabulary=LISTING_VIEWS,
        widget=MultiSelectionWidget(
            format='select',
            label='Available listing views',
            label_msgid='label_bookingcenter_available_listing_views',
            i18n_domain=I18N_DOMAIN,
            ),
        ),

    StringField(
        'defaultCalendarView',
        required=True,
        default='day',
        vocabulary=CALENDAR_VIEWS,
        widget=SelectionWidget(
            format='select',
            label='Default calendar view',
            label_msgid='label_bookingcenter_default_calendar_view',
            i18n_domain=I18N_DOMAIN,
            ),
        ),

    LinesField(
        'availableCalendarViews',
        required=True,
        default=('day', 'week', 'month'),
        vocabulary=CALENDAR_VIEWS,
        widget=MultiSelectionWidget(
            format='select',
            label='Available calendar views',
            label_msgid='label_bookingcenter_available_calendar_views',
            i18n_domain=I18N_DOMAIN,
            ),
        ),
                                                         
    IntegerField(
        'calendarStartingHour',
        required=True,
        default=8,
        widget=IntegerWidget(
            label='Sarting hour',
            description='This is used in day and week calendar views',
            label_msgid='label_bookingcenter_calendar_starting_hour',
            description_msgid='help_bookingcenter_calendar_starting_hour',
            i18n_domain=I18N_DOMAIN,
            ),
        ),
    
    IntegerField(
        'calendarEndingHour',
        required=True,
        default=19,
        widget=IntegerWidget(
            label='Ending hour',
            description='This is used in day and week calendar views',
            label_msgid='label_bookingcenter_calendar_ending_hour',
            description_msgid='help_bookingcenter_calendar_ending_hour',
            i18n_domain=I18N_DOMAIN,
            ),
        ),
                                                         
    StringField(
        'pendingColors',
        widget=StringWidget(
            label='Text and background colors for bookings in "pending" state',
            description='You must seperate the two colors with a ":" (example "red:white" where "red" is the\
                         text color and "white" is he background color)',
            label_msgid='label_pending_colors',
            description_msgid='help_pending_colors',
            i18n_domain=I18N_DOMAIN,
            ),
        ),
                                                         
    StringField(
        'bookedColors',
        widget=StringWidget(
            label='Text and background colors for bookings in "booked" state',
            description='You must seperate the two colors with a ":" (example "red:white") where "red" is the\
                         text color and "white" is the background color',
            label_msgid='label_booked_colors',
            description_msgid='help_booked_colors',
            i18n_domain=I18N_DOMAIN,
            ),
        ),
                                                         
    LinesField(
        'bookableObjectStates',
        default=('published',),
        widget=LinesWidget(
            label='Bookable object allowed states for book',
            description='Define here the workflow states of bookable objects you can book. If a bookable object is in a workflow\
                         state that is not in this list, it will not appear in available bookable objects.',
            label_msgid='label_bookable_object_states',
            description_msgid='help_bookable_object_states',
            i18n_domain=I18N_DOMAIN,
            ),
        ),
    
    StringField(
        'bookingReviewMode',
        default='review',
        vocabulary=GLOBAL_BOOKING_REVIEW_MODES,
        widget=SelectionWidget(
            label='Booking review mode',
            description='',
            label_msgid='label_booking_review_mode',
            description_msgid='',
            i18n_domain=I18N_DOMAIN,
            ),
        ),
))

BookingCenterSchema['description'].isMetadata = False
BookingCenterSchema['description'].schemata = 'default'

BookingSchema = BaseSchema.copy() + Schema((
    StringField(
        'fullName',
        required=True,
        default_method='getDefaultFullname',
        widget=StringWidget(
            label="Full name",
            label_msgid="label_booking_user_full_name",
            i18n_domain=I18N_DOMAIN,
            ),
        ),

    StringField(
        'phone',
        default_method='getDefaultPhone',
        required=True,
        widget=StringWidget(
            label="Phone",
            label_msgid="label_booking_user_phone",
            i18n_domain=I18N_DOMAIN,
            ),
        ),

    StringField(
        'email',
        required=True,
        default_method='getDefaultEmail',
        validators=('isEmail',),
        widget=StringWidget(
            label="Email",
            label_msgid="label_booking_user_email",
            i18n_domain=I18N_DOMAIN,
            ),
        ),

    DateTimeField(
        'startDate',
        required = True,
        accessor='start',
        default_method='getDefaultStartDate',
        languageIndependent=True,
        index='FieldIndex:brains',
        write_restricted_permission=CMFCorePermissions.ReviewPortalContent,
        widget=CalendarWidget(
            label="Booking start date",
            description=("Enter the starting date and time, or click "
                         "the calendar icon and select it. "),
            label_msgid="label_booking_start_date",
            description_msgid="help_booking_start_date",
            i18n_domain=I18N_DOMAIN,
            ),
        ),

    DateTimeField(
        'endDate',
        required = True,
        accessor='end',
        default_method='getDefaultEndDate',
        languageIndependent=True,
        write_restricted_permission=CMFCorePermissions.ReviewPortalContent,
        index='FieldIndex:brains',
        widget=CalendarWidget(
            label="Booking end date",
            description=("Enter the ending date and time, or click "
                         "the calendar icon and select it. "),                               
            label_msgid="label_booking_end_date",
            description_msgid="help_booking_end_date",
            i18n_domain=I18N_DOMAIN,
            ),
        ),
                                             
    StringField(
        'periodicityUID',
        default_method='getDefaultPeriodicityUID',
        languageIndependant=True,
        index='FieldIndex',
        widget=StringWidget(
            visible={'edit': 'invisible', 'view': 'invisible'},
            ),
        ),
  ))

field = BookingSchema['description']
BookingSchema.delField('description')
BookingSchema.addField(field)

BookingSchema['id'].widget.visible={'view' : 'hidden', 'edit' : 'hidden'}
BookingSchema['title'].required = False
BookingSchema['title'].widget.label='Booking title'
BookingSchema['title'].widget.label_msgid='label_booking_title'
BookingSchema['title'].widget.i18n_domain='plonebooking'
BookingSchema['description'].isMetadata = False
BookingSchema['description'].schemata = 'default'
BookingSchema['description'].widget.label='Comment'
BookingSchema['description'].widget.label_msgid='label_booking_description'
BookingSchema['description'].widget.i18n_domain='plonebooking'
BookingSchema['description'].widget.description=' '
BookingSchema['description'].widget.description_msgid='help_booking_description'

BookableObjectSchema = BaseSchema.copy() + Schema((
    StringField(
        'type',
        required=True,
        index='FieldIndex:brains',
        vocabulary='getTypeVocabulary',
        widget=SelectionWidget(
            format='select',
            label="Type",
            label_msgid="label_bookableobject_type",
            i18n_domain=I18N_DOMAIN,
            ),
        ),
                                                    
    StringField(
        'category',
        index='FieldIndex:brains',
        vocabulary='getCategoryVocabulary',
        widget=SelectionWidget(
            format='select',
            label="Category",
            label_msgid="label_bookableobject_category",
            i18n_domain=I18N_DOMAIN,
            ),
        ),
                                                    
    ComputedField(
        'bookedObjectUID',
        index='FieldIndex:brains',
        expression='context.getBookableObject().UID()', 
        widget=ComputedWidget(
            visible={'view': 'invisible', 'edit' : 'invisible'}, 
            ),
        ),
                                                    
    TextField(
        'text',
        default_content_type='text/html',
        default_output_type='text/html',
        allowable_content_types=('text/html',),
        widget=VisualWidget(
            label="Text",
            label_msgid="label_bookableobject_text",
            i18n_domain=I18N_DOMAIN,
            ),
        ),
                                                    
    StringField(
        'bookingReviewMode',
        vocabulary=BOOKING_REVIEW_MODES,
        default='default',
        widget=SelectionWidget(
            label='Booking review mode',
            description='',
            label_msgid='label_booking_review_mode',
            description_msgid='',
            i18n_domain=I18N_DOMAIN,
            ),
        ),
  ))

BookableObjectSchema['title'].widget.label = 'Bookable object title'
BookableObjectSchema['title'].widget.label_msgid = 'label_bookableobject_title'
BookableObjectSchema['title'].widget.i18n_domain = 'plonebooking'
BookableObjectSchema['description'].isMetadata = False
BookableObjectSchema['description'].schemata = 'default'